--[[
********************************
Licence: GPL
(c) msl 2013
v0.5.095
CHDK 1.3 r3200 or higher
optional iniLib is required
********************************
@title Backup Config
]]

cfg = {}
cfg[1] = require("gen/cnf_osd")
cfg[2] = require("gen/cnf_core")
cfg[3] = require("gen/cnf_user")
cfg[4] = require("gen/cnf_gps")

--language selection, returns 1 for English & 2 for German
lang = get_prop(get_propset() == 1 and 196 or 61)/256 == 1 and 2 or 1

--path & file name, read & write
cfg_path = "A/CHDK/CFG"
cfg_str  = {"BUP1", "BUP2", "BUP3", "BUP4"} --variable part of the file name

function rw_config(t, id)
    local message = {}
    message[1] = {"No Selection!", "%d File(s) saved.", "%d File(s) loaded."}
    message[2] = {"Keine Auswahl!", "%d Datei(en) gespeichert.", "%d Datei(en) geladen."}
    local cfg_type = {"OSD_", "COR_", "USE_", "GPS_"}
    local path = {}
    local ret = message[lang][1]
    if t[1] or t[2] or t[3] or t[4] then
        local i_end = GPS and 4 or 3
        local count = 0
        for i=1, i_end do
            path[i] = string.format("%s/%s%s.CFG", cfg_path, cfg_type[i], cfg_str[id])
            if t[5] then
                if t[i] then
                    save_config_file(cfg[i]._config_id, path[i])
                    count = count +1
                end
            end
            if t[6] then
                if t[i] then
                    if os.stat(path[i]) then
                        load_config_file(cfg[i]._config_id, path[i])
                        count = count +1
                    end
                end
            end
        end
        ret = string.format(t[5] and message[lang][2] or message[lang][3], count)
        draw_message(ret, WHITE)
    else
        draw_message(ret, RED)
    end
end

--UI
function cam_has_gps()
    local cams = "d20|s100|sx230hs|sx260hs|sx280hs"
    return string.find(cams, get_buildinfo().platform) ~= nil
end

function screen_w()
    local cams = "ixus110_sd960|ixus200_sd980|ixus240_elph320hs|ixus300_sd4000|ixus310_elph500hs|ixus1000_sd4500|sx1|sx210is|sx220hs|sx230hs"
    return string.find(cams, get_buildinfo().platform) and 480 or 360
end

GPS      = cam_has_gps()
SCREEN_W = screen_w()
SCREEN_R = SCREEN_W -1
SCREEN_H = 240
SCREEN_D = SCREEN_H -1 
FONT_W   = 8
BLACK    = 257
WHITE    = 258
RED      = 259
GREEN    = 262
BLUE     = 265
YELLOW   = 271

function draw_string_center(y, str, cl1, cl2)
    draw_string(SCREEN_W/2-#str*FONT_W/2, y, str, cl1, cl2)
end

function draw_message(str, cl)
    draw_rect_filled(3, 50, SCREEN_R-3, 196, BLACK, BLACK)
    draw_string_center(90, str, cl, BLACK)
    sleep(3000)
end

function draw_main_frame(title1, title2, keys, description)
    draw_rect_filled(0, 0, SCREEN_R, SCREEN_D, WHITE, BLACK, 2)
    draw_string(SCREEN_W/2-#title1*FONT_W, 4, title1, WHITE, BLACK, 2, 1)
    --draw_string_center( 4, title1, WHITE, BLACK)
    draw_string_center(20, title2, WHITE, BLACK)
    draw_line(0,  37, SCREEN_R,  37, WHITE)
    draw_line(0, 204, SCREEN_R, 204, WHITE)
    draw_string_center(207, keys       ,  BLUE, BLACK)
    draw_string_center(222, description, WHITE, BLACK)
end

function draw_user_frame()
    draw_rect_filled(3, 50, SCREEN_R-3, 196, BLACK, BLACK)
    local str = {}
    str[1] = {"CHDK OSD Elements", "Core Settings", "User Menu", "GPS", "Save", "Restore", "File Name: "}
    str[2] = {"CHDK-OSD-Elemente", "Allgemeine Einstellungen", "Benutzermen", "GPS", "Speichern", "Wiederherstellen", "Dateiname: "}
    for i=1, 7 do
        if not GPS and i==4 then i = i +1 end
        if i < 7 then
            draw_string(SCREEN_W/5, 30+20*i, toggle[i] and "[\149]" or "[ ]", BLUE , BLACK)
        end
        draw_string(SCREEN_W/3, i==7 and 180 or 30+20*i, str[lang][i], id==i and YELLOW or WHITE, BLACK)
    end
    draw_string(SCREEN_W/3 +#str[lang][7]*FONT_W, 180, "xxx_"..cfg_str[cfg_id], GREEN, BLACK)
end

function main_screen()
    local str1 = {}
    str1[1] = "CHDK Configuration"
    str1[2] = "CHDK-Konfiguration"
    local str2 = {}
    str2[1] = "save and restore"
    str2[2] = "sichern und wiederherstellen"
    local str3 = "  [\18]       [\29]     [SET]  [MENU]"
    local str4 = {}
    str4[1] = "Position Selection   OK     End"
    str4[2] = "Position  Auswahl    OK     Ende" 
    draw_main_frame(str1[lang], str2[lang], str3, str4[lang])
    local loop = true
    while loop do
        draw_user_frame()
        wait_click()
        if     is_pressed("set")  then rw_config(toggle, cfg_id)
        elseif is_pressed("up")   then
            id = id -1
            if not GPS and id == 4 then id = id -1 end
            if id < 1 then id = 7 end
        elseif is_pressed("down") then
            id = id +1
            if not GPS and id == 4 then id = id +1 end
            if id > 7 then id = 1 end
        elseif is_pressed("left") then
            if     id < 7  then toggle[id] = not toggle[id]
            elseif id == 7 then
                cfg_id = cfg_id -1
                if cfg_id < 1 then cfg_id = #cfg_str end
            end
        elseif is_pressed("right")then
            if     id < 7  then toggle[id] = not toggle[id]
            elseif id == 7 then
                cfg_id = cfg_id +1
                if cfg_id > #cfg_str then cfg_id = 1 end
            end
        elseif is_pressed("menu") then loop = false
        end 
        if id == 5 then toggle[6] = not toggle[5] end
        if id == 6 then toggle[5] = not toggle[6] end
    end
end

function write_ini()
    inicfg.toggle.osd    = toggle[1]
    inicfg.toggle.core   = toggle[2]
    inicfg.toggle.user   = toggle[3]
    inicfg.toggle.gps    = toggle[4]
    inicfg.toggle.save   = toggle[5]
    inicfg.toggle.recall = toggle[6]
    inicfg.pos.id        = id
    inicfg.pos.cfg_id    = cfg_id
    ini.write(inicfg)
end

function restore()
    if ini then write_ini() end
    cls()
    set_console_layout(0, 0, 25, 5)
    set_console_autoredraw(1)
    set_draw_title_line(1)
end

--MAIN--
if get_mode() == true then
    set_record(false)
    sleep(1000)
    while get_mode() do sleep(10) end
end

m_str = {}
m_str[1] = "CHDK Version 1.3 or higher required!!!"
m_str[2] = "CHDK-Version 1.3 oder hher bentigt!!!"
if tonumber(string.sub(string.gsub(get_buildinfo().build_number, "%p", ""), 0, 3)) >= 130 then
    if not os.stat(cfg_path) then os.mkdir(cfg_path) end
    toggle = {true, true, false, false, true, false}
    id     = 1
    cfg_id = 1
    if os.stat("A/CHDK/LUALIB/iniLib.lua") then
        ini =require("iniLib")
        inicfg, new = ini.read("cfg_bkup")
        if new then
            inicfg.toggle = {osd=true, core=true, user=false, gps=false, save=true, recall=false}
            inicfg.pos    = {id=1, cfg_id=1}
            ini.write(inicfg)
        end
        toggle[1] = inicfg.toggle.osd
        toggle[2] = inicfg.toggle.core
        toggle[3] = inicfg.toggle.user
        toggle[4] = inicfg.toggle.gps
        toggle[5] = inicfg.toggle.save
        toggle[6] = inicfg.toggle.recall
        id        = inicfg.pos.id
        cfg_id    = inicfg.pos.cfg_id
    end
    cls()
    set_console_layout(0, 0, 0, 0)
    set_console_autoredraw(0)
    set_draw_title_line(0)
    sleep(500) -- time for draw
    main_screen()
else
    draw_message(m_str[lang], RED)
end
restore()
